@@ -79,10 +79,11 @@ export PLATFORM_SHORT_ARCH
 
 # Add the NDK toolchain to the PATH, needed both for contribs and for building
 # stub libraries
-export PATH=${ANDROID_NDK}/toolchains/${PATH_HOST}-${GCCVER}/prebuilt/`uname|tr A-Z a-z`-x86/bin:${PATH}
+NDK_TOOLCHAIN_PATH=${ANDROID_NDK}/toolchains/${PATH_HOST}-${GCCVER}/prebuilt/`uname|tr A-Z a-z`-x86/bin
+export PATH=${NDK_TOOLCHAIN_PATH}:${PATH}
 
 # 1/ libvlc, libvlccore and its plugins
-TESTED_HASH=9aef14d5e5
+TESTED_HASH=4f42f23
 if [ ! -d "vlc" ]; then
     echo "VLC source not found, cloning"
     git clone git://git.videolan.org/vlc.git vlc
@@ -134,6 +135,11 @@ elif [ ${ANDROID_ABI} = "armeabi" ] ; then
     fi
 elif [ ${ANDROID_ABI} = "x86" ] ; then
     EXTRA_CFLAGS="-march=pentium -ffunction-sections -funwind-tables -frtti -fno-exceptions"
+elif [ ${ANDROID_ABI} = "mips" ] ; then
+    EXTRA_CFLAGS="-march=mips32 -mhard-float"
+    # All MIPS Linux kernels since 2.4.4 will trap any unimplemented FPU
+    # instruction and emulate it, so we select -mhard-float.
+    # See http://www.linux-mips.org/wiki/Floating_point#The_Linux_kernel_and_floating_point
 else
     echo "Unknown ABI. Die, die, die!"
     exit 2
@@ -219,3 +225,60 @@ export VLC_BUILD_DIR=vlc/android
 
 make distclean
 make -j1 TARGET_TUPLE=$TARGET_TUPLE PLATFORM_SHORT_ARCH=$PLATFORM_SHORT_ARCH CXXSTL=$CXXSTL $RELEASEFLAG
+
+# 3/ Environment script
+echo "Generating environment script."
+cat <<EOF
+This is a script that will export many of the variables used in this
+script. It will allow you to compile parts of the build without having
+to rebuild the entire build (e.g. recompile only the Java part).
+
+To use it, include the script into your shell, like this:
+    source env.sh
+
+Now, you can use this command to build the Java portion:
+    make -e
+
+The file will be automatically regenerated by compile.sh, so if you change
+your NDK/SDK locations or any build configurations, just re-run this
+script (sh compile.sh) and it will automatically update the file.
+
+EOF
+
+echo "# This file was automatically generated by compile.sh" > env.sh
+echo "# Re-run 'sh compile.sh' to update this file." >> env.sh
+
+# The essentials
+cat <<EssentialsA >> env.sh
+export ANDROID_ABI=$ANDROID_ABI
+export ANDROID_SDK=$ANDROID_SDK
+export ANDROID_NDK=$ANDROID_NDK
+export GCCVER=$GCCVER
+export CXXSTL=$CXXSTL
+export ANDROID_SYS_HEADERS_GINGERBREAD=$ANDROID_SYS_HEADERS_GINGERBREAD
+export ANDROID_SYS_HEADERS_HC=$ANDROID_SYS_HEADERS_HC
+export ANDROID_SYS_HEADERS_ICS=$ANDROID_SYS_HEADERS_ICS
+export ANDROID_LIBS=$ANDROID_LIBS
+export VLC_BUILD_DIR=$PWD/vlc/android
+export TARGET_TUPLE=$TARGET_TUPLE
+export PLATFORM_SHORT_ARCH=$PLATFORM_SHORT_ARCH
+EssentialsA
+
+# PATH
+echo "export PATH=$NDK_TOOLCHAIN_PATH:\${ANDROID_SDK}/platform-tools:\${PATH}" >> env.sh
+
+# CPU flags
+if [ -n "${HAVE_ARM}" ]; then
+    echo "export HAVE_ARM=1" >> env.sh
+elif [ -n "${HAVE_X86}" ]; then
+    echo "export HAVE_X86=1" >> env.sh
+elif [ -n "${HAVE_MIPS}" ]; then
+    echo "export HAVE_MIPS=1" >> env.sh
+fi
+
+if [ -n "${NO_ARMV6}" ]; then
+    echo "export NO_ARMV6=1" >> env.sh
+fi
+if [ -n "${NO_FPU}" ]; then
+    echo "export NO_FPU=1" >> env.sh
+fi
