@@ -40,6 +40,7 @@ import android.content.ComponentName;
 import android.content.Context;
 import android.content.Intent;
 import android.content.IntentFilter;
+import android.content.SharedPreferences;
 import android.graphics.Bitmap;
 import android.media.AudioManager;
 import android.media.AudioManager.OnAudioFocusChangeListener;
@@ -49,12 +50,12 @@ import android.media.RemoteControlClient.MetadataEditor;
 import android.os.Handler;
 import android.os.IBinder;
 import android.os.Message;
+import android.os.PowerManager;
 import android.os.RemoteException;
-import android.os.SystemClock;
+import android.preference.PreferenceManager;
 import android.support.v4.app.NotificationCompat;
 import android.telephony.TelephonyManager;
 import android.util.Log;
-import android.view.KeyEvent;
 
 public class AudioService extends Service {
 
@@ -62,10 +63,13 @@ public class AudioService extends Service {
 
     private static final int SHOW_PROGRESS = 0;
     public static final String START_FROM_NOTIFICATION = "from_notification";
-    public static final String ACTION_WIDGET_BACKWARD = "org.videolan.vlc.widget.Backward";
-    public static final String ACTION_WIDGET_PLAY = "org.videolan.vlc.widget.Play";
-    public static final String ACTION_WIDGET_STOP = "org.videolan.vlc.widget.Stop";
-    public static final String ACTION_WIDGET_FORWARD = "org.videolan.vlc.widget.Forward";
+    public static final String ACTION_REMOTE_GENERIC = "org.videolan.vlc.remote.";
+    public static final String ACTION_REMOTE_BACKWARD = "org.videolan.vlc.remote.Backward";
+    public static final String ACTION_REMOTE_PLAY = "org.videolan.vlc.remote.Play";
+    public static final String ACTION_REMOTE_PLAYPAUSE = "org.videolan.vlc.remote.PlayPause";
+    public static final String ACTION_REMOTE_PAUSE = "org.videolan.vlc.remote.Pause";
+    public static final String ACTION_REMOTE_STOP = "org.videolan.vlc.remote.Stop";
+    public static final String ACTION_REMOTE_FORWARD = "org.videolan.vlc.remote.Forward";
     public static final String ACTION_WIDGET_UPDATE = "org.videolan.vlc.widget.UPDATE";
 
     public static final String WIDGET_PACKAGE = "org.videolan.vlc";
@@ -80,14 +84,15 @@ public class AudioService extends Service {
     private boolean mShuffling = false;
     private RepeatType mRepeating = RepeatType.None;
     private boolean mDetectHeadset = true;
-    private long mHeadsetDownTime = 0;
-    private long mHeadsetUpTime = 0;
     private OnAudioFocusChangeListener audioFocusListener;
+    private ComponentName mRemoteControlClientReceiverComponent;
+    private PowerManager.WakeLock mWakeLock;
 
     /**
      * RemoteControlClient is for lock screen playback control.
      */
     private RemoteControlClient mRemoteControlClient = null;
+    private RemoteControlClientReceiver mRemoteControlClientReceiver = null;
 
     /**
      * Distinguish between the "fake" (Java-backed) playlist versus the "real"
@@ -115,47 +120,74 @@ public class AudioService extends Service {
         mMediaList = new ArrayList<Media>();
         mPrevious = new Stack<Media>();
         mEventManager = EventManager.getInstance();
+        mRemoteControlClientReceiverComponent = new ComponentName(getPackageName(),
+                RemoteControlClientReceiver.class.getName());
+
+        // Make sure the audio player will acquire a wake-lock while playing. If we don't do
+        // that, the CPU might go to sleep while the song is playing, causing playback to stop.
+        PowerManager pm = (PowerManager) getSystemService(Context.POWER_SERVICE);
+        mWakeLock = pm.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, TAG);
 
         IntentFilter filter = new IntentFilter();
         filter.setPriority(Integer.MAX_VALUE);
-        filter.addAction(ACTION_WIDGET_BACKWARD);
-        filter.addAction(ACTION_WIDGET_PLAY);
-        filter.addAction(ACTION_WIDGET_STOP);
-        filter.addAction(ACTION_WIDGET_FORWARD);
+        filter.addAction(ACTION_REMOTE_BACKWARD);
+        filter.addAction(ACTION_REMOTE_PLAYPAUSE);
+        filter.addAction(ACTION_REMOTE_PLAY);
+        filter.addAction(ACTION_REMOTE_PAUSE);
+        filter.addAction(ACTION_REMOTE_STOP);
+        filter.addAction(ACTION_REMOTE_FORWARD);
         filter.addAction(Intent.ACTION_HEADSET_PLUG);
         filter.addAction(AudioManager.ACTION_AUDIO_BECOMING_NOISY);
-        filter.addAction(Intent.ACTION_MEDIA_BUTTON);
         registerReceiver(serviceReceiver, filter);
 
-        if(Util.isICSOrLater())
-            setUpRemoteControlClient();
+        final SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(this);
+        boolean stealRemoteControl = pref.getBoolean("steal_remote_control", false);
+
+        if(!Util.isFroyoOrLater() || stealRemoteControl) {
+            /* Backward compatibility for API 7 */
+            filter = new IntentFilter();
+            if (stealRemoteControl)
+                filter.setPriority(Integer.MAX_VALUE);
+            filter.addAction(Intent.ACTION_MEDIA_BUTTON);
+            mRemoteControlClientReceiver = new RemoteControlClientReceiver();
+            registerReceiver(mRemoteControlClientReceiver, filter);
+        }
 
         AudioUtil.prepareCacheFolder(this);
     }
 
+
+    /**
+     * Set up the remote control and tell the system we want to be the default receiver for the MEDIA buttons
+     * @see http://android-developers.blogspot.fr/2010/06/allowing-applications-to-play-nicer.html
+     */
     @TargetApi(14)
-    private void setUpRemoteControlClient() {
-        if(!Util.isICSOrLater()) // sanity check
-            return;
+    public void setUpRemoteControlClient() {
+        Context context = VLCApplication.getAppContext();
+        AudioManager audioManager = (AudioManager)context.getSystemService(AUDIO_SERVICE);
 
-        AudioManager audioManager = (AudioManager)getSystemService(AUDIO_SERVICE);
-        ComponentName eventReceiverComponent = new ComponentName(getPackageName(), RemoteControlClientReceiver.class.getName());
+        if(Util.isICSOrLater()) {
+            audioManager.registerMediaButtonEventReceiver(mRemoteControlClientReceiverComponent);
 
-        audioManager.registerMediaButtonEventReceiver(eventReceiverComponent);
+            if (mRemoteControlClient == null) {
+                Intent mediaButtonIntent = new Intent(Intent.ACTION_MEDIA_BUTTON);
+                mediaButtonIntent.setComponent(mRemoteControlClientReceiverComponent);
+                PendingIntent mediaPendingIntent = PendingIntent.getBroadcast(context, 0, mediaButtonIntent, 0);
 
-        Intent mediaButtonIntent = new Intent(Intent.ACTION_MEDIA_BUTTON);
-        mediaButtonIntent.setComponent(eventReceiverComponent);
-        PendingIntent mediaPendingIntent = PendingIntent.getBroadcast(getApplicationContext(), 0, mediaButtonIntent, 0);
+                // create and register the remote control client
+                mRemoteControlClient = new RemoteControlClient(mediaPendingIntent);
+                audioManager.registerRemoteControlClient(mRemoteControlClient);
+            }
 
-        // create and register the remote control client
-        mRemoteControlClient = new RemoteControlClient(mediaPendingIntent);
-        mRemoteControlClient.setTransportControlFlags(
-                RemoteControlClient.FLAG_KEY_MEDIA_PLAY |
-                RemoteControlClient.FLAG_KEY_MEDIA_PAUSE |
-                RemoteControlClient.FLAG_KEY_MEDIA_PREVIOUS |
-                RemoteControlClient.FLAG_KEY_MEDIA_NEXT |
-                RemoteControlClient.FLAG_KEY_MEDIA_STOP);
-        audioManager.registerRemoteControlClient(mRemoteControlClient);
+            mRemoteControlClient.setTransportControlFlags(
+                    RemoteControlClient.FLAG_KEY_MEDIA_PLAY |
+                    RemoteControlClient.FLAG_KEY_MEDIA_PAUSE |
+                    RemoteControlClient.FLAG_KEY_MEDIA_PREVIOUS |
+                    RemoteControlClient.FLAG_KEY_MEDIA_NEXT |
+                    RemoteControlClient.FLAG_KEY_MEDIA_STOP);
+        } else if (Util.isFroyoOrLater()) {
+            audioManager.registerMediaButtonEventReceiver(mRemoteControlClientReceiverComponent);
+        }
     }
 
     /**
@@ -169,7 +201,8 @@ public class AudioService extends Service {
         if(!Util.isICSOrLater())
             return;
 
-        mRemoteControlClient.setPlaybackState(p);
+        if (mRemoteControlClient != null)
+            mRemoteControlClient.setPlaybackState(p);
     }
 
     @Override
@@ -181,7 +214,14 @@ public class AudioService extends Service {
     @Override
     public void onDestroy() {
         super.onDestroy();
+        stop();
+        if (mWakeLock.isHeld())
+            mWakeLock.release();
         unregisterReceiver(serviceReceiver);
+        if (mRemoteControlClientReceiver != null) {
+            unregisterReceiver(mRemoteControlClientReceiver);
+            mRemoteControlClientReceiver = null;
+        }
     }
 
     @Override
@@ -191,28 +231,21 @@ public class AudioService extends Service {
 
     @TargetApi(8)
     private void changeAudioFocus(boolean gain) {
-        if(!Util.isGingerbreadOrLater()) // NOP if not supported
+        if(!Util.isFroyoOrLater()) // NOP if not supported
             return;
 
         audioFocusListener = new OnAudioFocusChangeListener() {
-            int volume = -1;
             @Override
             public void onAudioFocusChange(int focusChange) {
-                AudioManager am = (AudioManager)getSystemService(AUDIO_SERVICE);
                 if(focusChange == AudioManager.AUDIOFOCUS_LOSS_TRANSIENT_CAN_DUCK ||
                    focusChange == AudioManager.AUDIOFOCUS_LOSS_TRANSIENT) {
-                    volume = am.getStreamVolume(AudioManager.STREAM_MUSIC);
-                    int maxVol = am.getStreamMaxVolume(AudioManager.STREAM_MUSIC);
                     /*
-                     * Lower the volume to 19% to "duck" when an alert or something
+                     * Lower the volume to 36% to "duck" when an alert or something
                      * needs to be played.
                      */
-                    am.setStreamVolume(AudioManager.STREAM_MUSIC, (int)(0.19*maxVol), 0);
+                    LibVLC.getExistingInstance().setVolume(36);
                 } else {
-                    if(volume != -1) {
-                        am.setStreamVolume(AudioManager.STREAM_MUSIC, volume, 0);
-                        volume = -1;
-                    }
+                    LibVLC.getExistingInstance().setVolume(100);
                 }
             }
         };
@@ -235,102 +268,41 @@ public class AudioService extends Service {
                 return;
             }
 
-            /*
-             * widget events
-             */
-            if (action.equalsIgnoreCase(ACTION_WIDGET_PLAY)) {
-                if (mLibVLC.isPlaying() && mCurrentMedia != null) {
-                    pause();
-                } else if (!mLibVLC.isPlaying() && mCurrentMedia != null) {
-                    play();
-                } else {
-                    Intent iVlc = new Intent(context, MainActivity.class);
-                    iVlc.putExtra(START_FROM_NOTIFICATION, true);
-                    iVlc.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
-                    context.startActivity(iVlc);
-                }
-            } else if (action.equalsIgnoreCase(ACTION_WIDGET_BACKWARD)) {
-                previous();
-            }
-            else if (action.equalsIgnoreCase(ACTION_WIDGET_STOP)) {
-                stop();
-            }
-            else if (action.equalsIgnoreCase(ACTION_WIDGET_FORWARD)) {
-                next();
-            }
-
             // skip all headsets events if there is a call
             TelephonyManager telManager = (TelephonyManager) getSystemService(Context.TELEPHONY_SERVICE);
             if (telManager != null && telManager.getCallState() != TelephonyManager.CALL_STATE_IDLE)
                 return;
 
             /*
-             * headset controller events
+             * Launch the activity if needed
              */
-            else if (action.equalsIgnoreCase(Intent.ACTION_MEDIA_BUTTON)) {
-                KeyEvent event = (KeyEvent) intent.getParcelableExtra(Intent.EXTRA_KEY_EVENT);
-                if (event == null)
-                    return;
+            if (action.startsWith(ACTION_REMOTE_GENERIC) && !mLibVLC.isPlaying() && mCurrentMedia == null) {
+                Intent iVlc = new Intent(context, MainActivity.class);
+                iVlc.putExtra(START_FROM_NOTIFICATION, true);
+                iVlc.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_SINGLE_TOP);
+                context.startActivity(iVlc);
+            }
 
-                if (mCurrentMedia == null) {
-                    abortBroadcast();
-                    return;
-                }
-
-                switch (event.getKeyCode())
-                {
-                /*
-                 * one click => play/pause
-                 * long click => previous
-                 * double click => next
-                 */
-                    case KeyEvent.KEYCODE_HEADSETHOOK:
-                    case KeyEvent.KEYCODE_MEDIA_PLAY_PAUSE:
-                        long time = SystemClock.uptimeMillis();
-                        switch (event.getAction())
-                        {
-                            case KeyEvent.ACTION_DOWN:
-                                if (event.getRepeatCount() > 0)
-                                    break;
-                                mHeadsetDownTime = time;
-                                break;
-                            case KeyEvent.ACTION_UP:
-                                // long click
-                                if (time - mHeadsetDownTime >= 1000) {
-                                    previous();
-                                    time = 0;
-                                    // double click
-                                } else if (time - mHeadsetUpTime <= 500) {
-                                    next();
-                                }
-                                // one click
-                                else {
-                                    if (mLibVLC.isPlaying())
-                                        pause();
-                                    else
-                                        play();
-                                }
-                                mHeadsetUpTime = time;
-                                break;
-                        }
-                        break;
-                    case KeyEvent.KEYCODE_MEDIA_PLAY:
-                        play();
-                        break;
-                    case KeyEvent.KEYCODE_MEDIA_PAUSE:
-                        pause();
-                        break;
-                    case KeyEvent.KEYCODE_MEDIA_STOP:
-                        stop();
-                        break;
-                    case KeyEvent.KEYCODE_MEDIA_NEXT:
-                        next();
-                        break;
-                    case KeyEvent.KEYCODE_MEDIA_PREVIOUS:
-                        previous();
-                        break;
-                }
-                abortBroadcast();
+            /*
+             * Remote / headset control events
+             */
+            if (action.equalsIgnoreCase(ACTION_REMOTE_PLAYPAUSE)) {
+                if (mLibVLC.isPlaying() && mCurrentMedia != null)
+                    pause();
+                else if (!mLibVLC.isPlaying() && mCurrentMedia != null)
+                    play();
+            } else if (action.equalsIgnoreCase(ACTION_REMOTE_PLAY)) {
+                if (!mLibVLC.isPlaying() && mCurrentMedia != null)
+                    play();
+            } else if (action.equalsIgnoreCase(ACTION_REMOTE_PAUSE)) {
+                if (mLibVLC.isPlaying() && mCurrentMedia != null)
+                    pause();
+            } else if (action.equalsIgnoreCase(ACTION_REMOTE_BACKWARD)) {
+                previous();
+            } else if (action.equalsIgnoreCase(ACTION_REMOTE_STOP)) {
+                stop();
+            } else if (action.equalsIgnoreCase(ACTION_REMOTE_FORWARD)) {
+                next();
             }
 
             /*
@@ -371,6 +343,8 @@ public class AudioService extends Service {
                     Log.i(TAG, "MediaPlayerPlaying");
                     service.changeAudioFocus(true);
                     service.setRemoteControlClientPlaybackState(RemoteControlClient.PLAYSTATE_PLAYING);
+                    if (!service.mWakeLock.isHeld())
+                        service.mWakeLock.acquire();
                     break;
                 case EventManager.MediaPlayerPaused:
                     Log.i(TAG, "MediaPlayerPaused");
@@ -378,16 +352,22 @@ public class AudioService extends Service {
                     // also hide notification if phone ringing
                     service.hideNotification();
                     service.setRemoteControlClientPlaybackState(RemoteControlClient.PLAYSTATE_PAUSED);
+                    if (service.mWakeLock.isHeld())
+                        service.mWakeLock.release();
                     break;
                 case EventManager.MediaPlayerStopped:
                     Log.i(TAG, "MediaPlayerStopped");
                     service.executeUpdate();
                     service.setRemoteControlClientPlaybackState(RemoteControlClient.PLAYSTATE_STOPPED);
+                    if (service.mWakeLock.isHeld())
+                        service.mWakeLock.release();
                     break;
                 case EventManager.MediaPlayerEndReached:
                     Log.i(TAG, "MediaPlayerEndReached");
                     service.executeUpdate();
                     service.next();
+                    if (service.mWakeLock.isHeld())
+                        service.mWakeLock.release();
                     break;
                 case EventManager.MediaPlayerVout:
                     if(msg.getData().getInt("data") > 0) {
@@ -460,6 +440,7 @@ public class AudioService extends Service {
             .setSmallIcon(R.drawable.icon)
             .setLargeIcon(AudioUtil.getCover(this, mCurrentMedia, 64))
             .setContentTitle(mCurrentMedia.getTitle())
+            .setTicker(mCurrentMedia.getTitle() + " - " + mCurrentMedia.getArtist())
             .setContentText((Util.isJellyBeanOrLater() ? mCurrentMedia.getArtist()
                     : mCurrentMedia.getArtist() + " - " + mCurrentMedia.getAlbum()))
                     .setContentInfo(mCurrentMedia.getAlbum())
@@ -487,6 +468,7 @@ public class AudioService extends Service {
     }
 
     private void pause() {
+        setUpRemoteControlClient();
         mHandler.removeMessages(SHOW_PROGRESS);
         // hideNotification(); <-- see event handler
         mLibVLC.pause();
@@ -494,6 +476,7 @@ public class AudioService extends Service {
 
     private void play() {
         if (mCurrentMedia != null) {
+            setUpRemoteControlClient();
             mLibVLC.play();
             mHandler.sendEmptyMessage(SHOW_PROGRESS);
             showNotification();
@@ -526,7 +509,7 @@ public class AudioService extends Service {
         } else if (!mShuffling && index < mMediaList.size() - 1) {
             mCurrentMedia = mMediaList.get(index + 1);
         } else {
-            if (mRepeating == RepeatType.All)
+            if (mRepeating == RepeatType.All && mMediaList.size() > 0)
                 mCurrentMedia = mMediaList.get(0);
             else {
                 stop();
@@ -544,6 +527,7 @@ public class AudioService extends Service {
             mLibVLC.readMedia(mCurrentMedia.getLocation(), true);
         }
         mHandler.sendEmptyMessage(SHOW_PROGRESS);
+        setUpRemoteControlClient();
         showNotification();
         updateWidget(this);
         updateRemoteControlClientMetadata();
@@ -554,14 +538,16 @@ public class AudioService extends Service {
         if(!Util.isICSOrLater()) // NOP check
             return;
 
-        MetadataEditor editor = mRemoteControlClient.editMetadata(true);
-        editor.putString(MediaMetadataRetriever.METADATA_KEY_ALBUM, mCurrentMedia.getAlbum());
-        editor.putString(MediaMetadataRetriever.METADATA_KEY_ARTIST, mCurrentMedia.getArtist());
-        editor.putString(MediaMetadataRetriever.METADATA_KEY_GENRE, mCurrentMedia.getGenre());
-        editor.putString(MediaMetadataRetriever.METADATA_KEY_TITLE, mCurrentMedia.getTitle());
-        editor.putLong(MediaMetadataRetriever.METADATA_KEY_DURATION, mCurrentMedia.getLength());
-        editor.putBitmap(MetadataEditor.BITMAP_KEY_ARTWORK, getCover());
-        editor.apply();
+        if (mRemoteControlClient != null) {
+            MetadataEditor editor = mRemoteControlClient.editMetadata(true);
+            editor.putString(MediaMetadataRetriever.METADATA_KEY_ALBUM, mCurrentMedia.getAlbum());
+            editor.putString(MediaMetadataRetriever.METADATA_KEY_ARTIST, mCurrentMedia.getArtist());
+            editor.putString(MediaMetadataRetriever.METADATA_KEY_GENRE, mCurrentMedia.getGenre());
+            editor.putString(MediaMetadataRetriever.METADATA_KEY_TITLE, mCurrentMedia.getTitle());
+            editor.putLong(MediaMetadataRetriever.METADATA_KEY_DURATION, mCurrentMedia.getLength());
+            editor.putBitmap(MetadataEditor.BITMAP_KEY_ARTWORK, getCover());
+            editor.apply();
+        }
     }
 
     private void previous() {
@@ -583,6 +569,7 @@ public class AudioService extends Service {
             mLibVLC.readMedia(mCurrentMedia.getLocation(), true);
         }
         mHandler.sendEmptyMessage(SHOW_PROGRESS);
+        setUpRemoteControlClient();
         showNotification();
         updateWidget(this);
         updateRemoteControlClientMetadata();
@@ -745,6 +732,7 @@ public class AudioService extends Service {
                 } else {
                     mLibVLC.readMedia(mCurrentMedia.getLocation());
                 }
+                setUpRemoteControlClient();
                 showNotification();
                 updateWidget(AudioService.this);
                 updateRemoteControlClientMetadata();
