@@ -201,7 +201,7 @@ public class LibVLC {
         if (!mIsInitialized) {
             if(!Util.hasCompatibleCPU()) {
                 Log.e(TAG, Util.getErrorMsg());
-                return;
+                throw new LibVlcException();
             }
             Context context = VLCApplication.getAppContext();
             SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(context);
@@ -239,6 +239,15 @@ public class LibVLC {
         mAout.playBuffer(audioData, bufferSize);
     }
 
+    /**
+     * Pause the Java audio output
+     * This function is called by the native code
+     */
+    public void pauseAout() {
+        Log.d(TAG, "Pausing the java audio output");
+        mAout.pause();
+    }
+
     /**
      * Close the Java audio output
      * This function is called by the native code
@@ -387,7 +396,7 @@ public class LibVLC {
     public native int getVolume();
 
     /**
-     * Gets volume as integer
+     * Sets volume as integer
      * @param volume: Volume level passed as integer
      */
     public native int setVolume(int volume);
