@@ -23,15 +23,18 @@ package org.videolan.vlc;
 import java.io.BufferedReader;
 import java.io.ByteArrayInputStream;
 import java.io.File;
+import java.io.FileNotFoundException;
 import java.io.FileReader;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.text.DecimalFormat;
 import java.text.NumberFormat;
+import java.util.ArrayList;
 import java.util.HashSet;
 import java.util.Locale;
 import java.util.Properties;
+import java.util.StringTokenizer;
 
 import android.content.Context;
 import android.graphics.Bitmap;
@@ -47,7 +50,7 @@ import android.widget.Toast;
 
 public class Util {
     public final static String TAG = "VLC/Util";
-    public final static boolean hasNavBar;
+    private final static boolean hasNavBar;
     /** A set of utility functions for the VLC application */
 
     static {
@@ -253,6 +256,13 @@ public class Util {
         return hasNavBar;
     }
 
+    /** hasCombBar test if device has Combined Bar : only for tablet with Honeycomb or ICS */
+    public static boolean hasCombBar() {
+        return (!isPhone()
+                && ((android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.HONEYCOMB) &&
+                    (android.os.Build.VERSION.SDK_INT <= android.os.Build.VERSION_CODES.JELLY_BEAN)));
+    }
+
     private static String errorMsg = null;
     private static boolean isCompatible = false;
     public static String getErrorMsg() {
@@ -285,7 +295,8 @@ public class Util {
         String ANDROID_ABI = properties.getProperty("ANDROID_ABI");
         boolean NO_FPU = properties.getProperty("NO_FPU","0").equals("1");
         boolean NO_ARMV6 = properties.getProperty("NO_ARMV6","0").equals("1");
-        boolean hasNeon = false, hasFpu = false, hasArmV6 = false, hasArmV7 = false;
+        boolean hasNeon = false, hasFpu = false, hasArmV6 = false,
+                hasArmV7 = false, hasMips = false;
         boolean hasX86 = false;
 
         if(CPU_ABI.equals("x86")) {
@@ -314,7 +325,10 @@ public class Util {
                 // (see kernel sources arch/x86/kernel/cpu/proc.c)
                 if(line.contains("clflush size"))
                     hasX86 = true;
-                // TODO: MIPS - "microsecond timers"; see arch/mips/kernel/proc.c
+                // "microsecond timers" is specific to MIPS.
+                // see arch/mips/kernel/proc.c
+                if(line.contains("microsecond timers"))
+                    hasMips = true;
                 if(!hasNeon && line.contains("neon"))
                     hasNeon = true;
                 if(!hasFpu && line.contains("vfp"))
@@ -339,6 +353,16 @@ public class Util {
             return false;
         }
 
+        if(ANDROID_ABI.equals("mips") && !hasMips) {
+            errorMsg = "MIPS build on non-MIPS device";
+            isCompatible = false;
+            return false;
+        } else if(hasMips && ANDROID_ABI.contains("armeabi")) {
+            errorMsg = "ARM build on MIPS device";
+            isCompatible = false;
+            return false;
+        }
+
         if(ANDROID_ABI.equals("armeabi-v7a") && !hasArmV7) {
             errorMsg = "ARMv7 build on non-ARMv7 device";
             isCompatible = false;
@@ -373,4 +397,53 @@ public class Util {
             return true;
         }
     }
+
+    public static String[] getStorageDirectories()
+    {
+        String[] dirs = null;
+        BufferedReader bufReader = null;
+        try {
+            bufReader = new BufferedReader(new FileReader("/proc/mounts"));
+            ArrayList<String> list = new ArrayList<String>();
+            list.add(Environment.getExternalStorageDirectory().getPath());
+            String line;
+            while ((line = bufReader.readLine()) != null) {
+                if (line.contains("vfat") || line.contains("/mnt")) {
+                    StringTokenizer tokens = new StringTokenizer(line, " ");
+                    String s = tokens.nextToken();
+                    s = tokens.nextToken(); // Take the second token, i.e. mount point
+
+                    if (list.contains(s))
+                        continue;
+
+                    if (line.contains("/dev/block/vold")) {
+                        if (!line.startsWith("/mnt/secure") &&
+                            !line.startsWith("/mnt/shell") &&
+                            !line.startsWith("/mnt/asec") &&
+                            !line.startsWith("/mnt/obb") &&
+                            !line.startsWith("/dev/mapper") &&
+                            !line.startsWith("tmpfs")) {
+                            list.add(s);
+                        }
+                    }
+                }
+            }
+
+            dirs = new String[list.size()];
+            for (int i = 0; i < list.size(); i++) {
+                dirs[i] = list.get(i);
+            }
+        }
+        catch (FileNotFoundException e) {}
+        catch (IOException e) {}
+        finally {
+            if (bufReader != null) {
+                try {
+                    bufReader.close();
+                }
+                catch (IOException e) {}
+            }
+        }
+        return dirs;
+    }
 }
