@@ -29,9 +29,11 @@ import org.videolan.vlc.R;
 import org.videolan.vlc.Util;
 
 import android.app.ListActivity;
-import android.content.SharedPreferences;
+import android.content.BroadcastReceiver;
+import android.content.Context;
+import android.content.Intent;
+import android.content.IntentFilter;
 import android.os.Bundle;
-import android.preference.PreferenceManager;
 import android.view.KeyEvent;
 import android.view.View;
 import android.widget.ListView;
@@ -46,7 +48,7 @@ public class BrowserActivity extends ListActivity {
     private BrowserAdapter mAdapter;
     private File mCurrentDir;
     private final Stack<ScrollState> mScollStates = new Stack<ScrollState>();
-    private String mRoot;
+    private String mRoots[];
 
     private class ScrollState {
         public ScrollState(int index, int top) {
@@ -65,26 +67,40 @@ public class BrowserActivity extends ListActivity {
         mAdapter = new BrowserAdapter(this);
         setListAdapter(mAdapter);
 
-        //get the root from the settings
-        SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(this);
-        mRoot = pref.getString("directories_root", "/");
+        IntentFilter filter = new IntentFilter();
+        filter.addAction(Intent.ACTION_MEDIA_MOUNTED);
+        filter.addAction(Intent.ACTION_MEDIA_UNMOUNTED);
+        filter.addAction(Intent.ACTION_MEDIA_REMOVED);
+        filter.addAction(Intent.ACTION_MEDIA_EJECT);
+        filter.addDataScheme("file");
+        registerReceiver(messageReceiver, filter);
 
-        //Make sure the path is valid, use "/" if it is not
-        File file = new File(mRoot);
-        if (!file.exists())
-            file = new File("/");
-        mRoot = file.getPath();
-
-        openDir(file);
+        mRoots = Util.getStorageDirectories();
+        openStorageDevices(mRoots);
     }
 
     @Override
     protected void onDestroy() {
         super.onDestroy();
+        unregisterReceiver(messageReceiver);
         mAdapter.clear();
         mScollStates.clear();
     }
 
+    private void openStorageDevices(String roots[]) {
+        mCurrentDir = null;
+        mAdapter.clear();
+        for (String s : roots) {
+            File f = new File(s);
+            if (f.exists())
+                mAdapter.add(f);
+        }
+        mAdapter.sort();
+
+        // set scroll position to top
+        getListView().setSelection(0);
+    }
+
     private void openDir(File file) {
         if (!file.exists() || file.getPath() == null)
             return;
@@ -124,8 +140,23 @@ public class BrowserActivity extends ListActivity {
     @Override
     public boolean onKeyDown(int keyCode, KeyEvent event) {
         if (keyCode == KeyEvent.KEYCODE_BACK) {
-            if (mCurrentDir.getPath().equals(mRoot)) {
+            if (mCurrentDir == null) {
+                // We're on the list of storage devices
                 return super.onKeyDown(keyCode, event);
+            }
+
+            // Check if we are on one of the root
+            boolean isRoot = false;
+            for (String root: mRoots) {
+                if (mCurrentDir.getPath().equals(root)) {
+                    isRoot = true;
+                    break;
+                }
+            }
+
+            if (isRoot) {
+                openStorageDevices(mRoots);
+                return true;
             } else {
                 openDir(mCurrentDir.getParentFile());
                 // restore scroll state
@@ -137,6 +168,30 @@ public class BrowserActivity extends ListActivity {
         return super.onKeyDown(keyCode, event);
     }
 
+    public void refresh() {
+        if (mCurrentDir == null) {
+            mRoots = Util.getStorageDirectories();
+            openStorageDevices(mRoots);
+        } else {
+            openDir(mCurrentDir);
+        }
+        mAdapter.notifyDataSetChanged();
+    }
+
+    private final BroadcastReceiver messageReceiver = new BroadcastReceiver() {
+        @Override
+        public void onReceive(Context context, Intent intent) {
+            String action = intent.getAction();
+
+            if (action.equalsIgnoreCase(Intent.ACTION_MEDIA_MOUNTED) ||
+                action.equalsIgnoreCase(Intent.ACTION_MEDIA_UNMOUNTED) ||
+                action.equalsIgnoreCase(Intent.ACTION_MEDIA_REMOVED) ||
+                action.equalsIgnoreCase(Intent.ACTION_MEDIA_EJECT)) {
+                refresh();
+            }
+        }
+    };
+
     /**
      * Filter: accept only directories
      */
