@@ -34,7 +34,7 @@ import org.videolan.vlc.Util;
 import org.videolan.vlc.VLCApplication;
 
 import android.content.Context;
-import android.preference.PreferenceManager;
+import android.os.Environment;
 import android.util.Log;
 import android.view.LayoutInflater;
 import android.view.View;
@@ -56,10 +56,16 @@ public class DirectoryAdapter extends BaseAdapter {
     public class Node implements Comparable<DirectoryAdapter.Node> {
         public ArrayList<DirectoryAdapter.Node> children;
         String name;
+        String visibleName;
         public Boolean isFile;
 
         public Node(String _name) {
+            this(_name, null);
+        }
+
+        public Node(String _name, String _visibleName) {
             this.name = _name;
+            this.visibleName = _visibleName;
             this.children = new ArrayList<DirectoryAdapter.Node>();
             this.isFile = false;
         }
@@ -104,6 +110,10 @@ public class DirectoryAdapter extends BaseAdapter {
             return c;
         }
 
+        public String getVisibleName() {
+            return (this.visibleName != null) ? this.visibleName : this.name;
+        }
+
         @Override
         public int compareTo(Node arg0) {
             if(this.isFile && !(arg0.isFile))
@@ -123,6 +133,20 @@ public class DirectoryAdapter extends BaseAdapter {
     }
 
     public void populateNode(DirectoryAdapter.Node n, String path) {
+        if (path == null) {
+            // We're on the storage list
+            String storages[] = Util.getStorageDirectories();
+            for (String storage : storages) {
+                File f = new File(storage);
+                DirectoryAdapter.Node child = new DirectoryAdapter.Node(f.getName(), getVisibleName(f));
+                child.isFile = false;
+                this.populateNode(child, storage);
+                n.children.add(child);
+            }
+            return;
+        }
+
+
         File file = new File(path);
         if(!file.exists() || !file.isDirectory())
             return;
@@ -169,37 +193,28 @@ public class DirectoryAdapter extends BaseAdapter {
             }
             Collections.sort(n.children);
         }
-        /* Don't let the user escape into the wild by jumping above the root dir */
-        if(mCurrentDir.contains(mRootDir) && !mCurrentDir.equals(mRootDir)) {
-            DirectoryAdapter.Node up = new DirectoryAdapter.Node("..");
-            n.children.add(0, up);
-        }
+
+        DirectoryAdapter.Node up = new DirectoryAdapter.Node("..");
+        n.children.add(0, up);
     }
 
     private LayoutInflater mInflater;
     private DirectoryAdapter.Node mRootNode;
     private DirectoryAdapter.Node mCurrentNode;
-    private String mRootDir;
     private String mCurrentDir;
+    private String mCurrentRoot;
 
     public DirectoryAdapter() {
-        DirectoryAdapter_Core(
-                PreferenceManager.getDefaultSharedPreferences(VLCApplication.getAppContext())
-                .getString("directories_root", android.os.Environment.getExternalStorageDirectory().getPath())
-        );
-    }
-
-    public DirectoryAdapter(Context context, String rootDir) {
-        DirectoryAdapter_Core(rootDir);
+        DirectoryAdapter_Core(null);
     }
 
     private void DirectoryAdapter_Core(String rootDir) {
-        rootDir = Util.stripTrailingSlash(rootDir);
+        if (rootDir != null)
+            rootDir = Util.stripTrailingSlash(rootDir);
         Log.v(TAG, "rootMRL is " + rootDir);
         mInflater = LayoutInflater.from(VLCApplication.getAppContext());
         mRootNode = new DirectoryAdapter.Node(rootDir);
         mCurrentDir = rootDir;
-        mRootDir = rootDir;
         this.populateNode(mRootNode, rootDir);
         mCurrentNode = mRootNode;
     }
@@ -256,25 +271,25 @@ public class DirectoryAdapter extends BaseAdapter {
             holder.title.setText(m.getTitle());
             holderText = m.getArtist() + " - " + m.getAlbum();
         } else
-            holder.title.setText(selectedNode.name);
+            holder.title.setText(selectedNode.getVisibleName());
 
         if(selectedNode.name == "..")
             holderText = context.getString(R.string.parent_folder);
         else if(!selectedNode.isFile()) {
             int folderCount = selectedNode.subfolderCount();
-            int songCount = selectedNode.subfilesCount();
+            int mediaFileCount = selectedNode.subfilesCount();
             holderText = "";
 
             if(folderCount > 0)
                 holderText += context.getResources().getQuantityString(
                         R.plurals.subfolders_quantity, folderCount, folderCount
                 );
-            if(folderCount > 0 && songCount > 0)
+            if(folderCount > 0 && mediaFileCount > 0)
                 holderText += ", ";
-            if(songCount > 0)
+            if(mediaFileCount > 0)
                 holderText += context.getResources().getQuantityString(
-                        R.plurals.songs_quantity, songCount, songCount
-                );
+                        R.plurals.mediafiles_quantity, mediaFileCount,
+                        mediaFileCount);
         }
         holder.text.setText(holderText);
         if(selectedNode.isFile())
@@ -292,14 +307,33 @@ public class DirectoryAdapter extends BaseAdapter {
     }
 
     public boolean browse(String directoryName) {
-        try {
-            this.mCurrentDir = new URI(
-                    Util.PathToURI(this.mCurrentDir + "/" + directoryName))
-                    .normalize().getPath();
-            this.mCurrentDir = Util.stripTrailingSlash(this.mCurrentDir);
-        } catch (URISyntaxException e) {
-            e.printStackTrace();
-            return false;
+        if (this.mCurrentDir == null) {
+            // We're on the storage list
+            String storages[] = Util.getStorageDirectories();
+            for (String storage : storages) {
+                storage = Util.stripTrailingSlash(storage);
+                if (storage.endsWith(directoryName)) {
+                    this.mCurrentRoot = storage;
+                    this.mCurrentDir = storage;
+                    this.mCurrentDir = Util.stripTrailingSlash(this.mCurrentDir);
+                    break;
+                }
+            }
+        } else {
+            try {
+                this.mCurrentDir = new URI(
+                        Util.PathToURI(this.mCurrentDir + "/" + directoryName))
+                        .normalize().getPath();
+                this.mCurrentDir = Util.stripTrailingSlash(this.mCurrentDir);
+                if (this.mCurrentDir.equals(getParentDir(this.mCurrentRoot))) {
+                    // Returning on the storage list
+                    this.mCurrentDir = null;
+                    this.mCurrentRoot = null;
+                }
+            } catch (URISyntaxException e) {
+                e.printStackTrace();
+                return false;
+            }
         }
 
         Log.d(TAG, "Browsing to " + this.mCurrentDir);
@@ -325,8 +359,8 @@ public class DirectoryAdapter extends BaseAdapter {
         );
     }
 
-    public String getmRootDir() {
-        return mRootDir;
+    public boolean isRoot() {
+        return mCurrentDir == null;
     }
 
     public String getmCurrentDir() {
@@ -351,4 +385,24 @@ public class DirectoryAdapter extends BaseAdapter {
 
         this.notifyDataSetChanged();
     }
+
+    private String getParentDir(String path) {
+        try {
+            path = new URI(Util.PathToURI(path + "/.."))
+                    .normalize().getPath();
+        } catch (URISyntaxException e) {
+            e.printStackTrace();
+        }
+        return Util.stripTrailingSlash(path);
+    }
+
+    private String getVisibleName(File file) {
+        if (android.os.Build.VERSION.SDK_INT >= 17) {
+            // Show "sdcard" for the user's folder when running in multi-user
+            if (file.getAbsolutePath().equals(Environment.getExternalStorageDirectory().getPath())) {
+                return VLCApplication.getAppContext().getString(R.string.internal_memory);
+            }
+        }
+        return file.getName();
+    }
 }
