@@ -33,6 +33,10 @@ import org.videolan.vlc.gui.video.VideoPlayerActivity;
 import org.videolan.vlc.interfaces.ISortable;
 
 import android.app.AlertDialog;
+import android.content.BroadcastReceiver;
+import android.content.Context;
+import android.content.Intent;
+import android.content.IntentFilter;
 import android.os.Bundle;
 import android.view.ContextMenu;
 import android.view.ContextMenu.ContextMenuInfo;
@@ -61,6 +65,14 @@ public class DirectoryViewFragment extends SherlockListFragment implements ISort
         super.onCreate(savedInstanceState);
 
         mDirectoryAdapter = new DirectoryAdapter();
+
+        IntentFilter filter = new IntentFilter();
+        filter.addAction(Intent.ACTION_MEDIA_MOUNTED);
+        filter.addAction(Intent.ACTION_MEDIA_UNMOUNTED);
+        filter.addAction(Intent.ACTION_MEDIA_REMOVED);
+        filter.addAction(Intent.ACTION_MEDIA_EJECT);
+        filter.addDataScheme("file");
+        getActivity().registerReceiver(messageReceiver, filter);
     }
 
     @Override
@@ -85,6 +97,12 @@ public class DirectoryViewFragment extends SherlockListFragment implements ISort
         return v;
     }
 
+    @Override
+    public void onDestroy() {
+        super.onDestroy();
+        getActivity().unregisterReceiver(messageReceiver);
+    }
+
     @Override
     public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
         int position = ((AdapterContextMenuInfo)menuInfo).position;
@@ -145,12 +163,8 @@ public class DirectoryViewFragment extends SherlockListFragment implements ISort
     }
 
     public boolean isRootDirectory () {
-        if (mDirectoryAdapter.getmCurrentDir() != null &&
-            mDirectoryAdapter.getmCurrentDir().equals(mDirectoryAdapter.getmRootDir())) {
-            return true;
-        }
-        return false;
-    };
+        return mDirectoryAdapter.isRoot();
+    }
 
     public void showParentDirectory() {
             mDirectoryAdapter.browse("..");
@@ -183,4 +197,18 @@ public class DirectoryViewFragment extends SherlockListFragment implements ISort
         if (mDirectoryAdapter != null)
             mDirectoryAdapter.refresh();
     }
+
+    private final BroadcastReceiver messageReceiver = new BroadcastReceiver() {
+        @Override
+        public void onReceive(Context context, Intent intent) {
+            String action = intent.getAction();
+
+            if (action.equalsIgnoreCase(Intent.ACTION_MEDIA_MOUNTED) ||
+                action.equalsIgnoreCase(Intent.ACTION_MEDIA_UNMOUNTED) ||
+                action.equalsIgnoreCase(Intent.ACTION_MEDIA_REMOVED) ||
+                action.equalsIgnoreCase(Intent.ACTION_MEDIA_EJECT)) {
+                refresh();
+            }
+        }
+    };
 }
