@@ -20,9 +20,7 @@
 
 package org.videolan.vlc.gui;
 
-import java.io.File;
-import java.util.ArrayList;
-
+import org.videolan.vlc.AudioService;
 import org.videolan.vlc.AudioServiceController;
 import org.videolan.vlc.DatabaseManager;
 import org.videolan.vlc.LibVLC;
@@ -31,12 +29,11 @@ import org.videolan.vlc.R;
 import org.videolan.vlc.Util;
 
 import android.app.AlertDialog;
-import android.app.Dialog;
+import android.content.Context;
 import android.content.DialogInterface;
 import android.content.Intent;
 import android.content.SharedPreferences;
 import android.os.Bundle;
-import android.os.Environment;
 import android.preference.CheckBoxPreference;
 import android.preference.EditTextPreference;
 import android.preference.ListPreference;
@@ -46,14 +43,9 @@ import android.preference.Preference.OnPreferenceClickListener;
 import android.preference.PreferenceActivity;
 import android.preference.PreferenceManager;
 import android.util.Log;
-import android.view.View;
-import android.widget.Button;
-import android.widget.CompoundButton;
-import android.widget.RadioButton;
-import android.widget.RadioGroup;
-import android.widget.TextView;
 import android.widget.Toast;
 
+@SuppressWarnings("deprecation")
 public class PreferencesActivity extends PreferenceActivity {
 
     public final static String TAG = "VLC/PreferencesActivity";
@@ -62,124 +54,11 @@ public class PreferencesActivity extends PreferenceActivity {
     public final static String LAST_MEDIA = "LastMedia";
     public final static int RESULT_RESCAN = RESULT_FIRST_USER + 1;
 
-    @SuppressWarnings("deprecation")
     @Override
     protected void onCreate(Bundle savedInstanceState) {
         super.onCreate(savedInstanceState);
         addPreferencesFromResource(R.xml.preferences);
 
-        // Create onPrefChange
-        Preference rootDirectoryPref = findPreference("directories_root");
-        rootDirectoryPref.setOnPreferenceClickListener(new OnPreferenceClickListener() {
-
-            @Override
-            public boolean onPreferenceClick(Preference arg0) {
-                final SharedPreferences sharedPrefs = PreferenceManager.getDefaultSharedPreferences(PreferencesActivity.this);
-                final String current_path = sharedPrefs.getString("directories_root",Environment.getExternalStorageDirectory().getAbsolutePath());
-
-                final Dialog dialog = new Dialog(PreferencesActivity.this);
-                dialog.setContentView(R.layout.root_selection);
-                dialog.setCancelable(true);
-                dialog.setTitle(R.string.filebrowser_root);
-
-                ArrayList<String> extMounts = new ArrayList<String>();
-                File mntDir = new File("/mnt");
-                String[] mntDirSubdirs = mntDir.list();
-                if(mntDirSubdirs != null) {
-                    for(String s : mntDirSubdirs) {
-                        if(s == null) continue;
-
-                        // skip over irrelevant mounts
-                        if(s.equals("asec") || s.equals("obb") || s.equals("secure") || s.equals("sdcard")) continue;
-
-                        if(new File("/mnt/" + s).isDirectory()) {
-                            extMounts.add("/mnt/" + s);
-                        }
-                    }
-                }
-
-                Boolean selected = false;
-
-                RadioButton internal = (RadioButton)dialog.findViewById(R.id.internal_memory);
-                internal.setTag(Environment.getExternalStorageDirectory().getAbsolutePath());
-                final TextView other_path = (TextView)dialog.findViewById(R.id.other_path);
-                other_path.setEnabled(false); //enable only when "other" is selected
-                RadioButton other = (RadioButton)dialog.findViewById(R.id.other);
-                other.setOnCheckedChangeListener(new RadioButton.OnCheckedChangeListener() {
-                    @Override
-                    public void onCheckedChanged(CompoundButton arg0, boolean isClicked) {
-                        other_path.setEnabled(isClicked);
-                        if(isClicked)
-                            other_path.requestFocus();
-                        else
-                            other_path.clearFocus();
-                    }
-                });
-
-                // fill in the radio buttons
-                if(current_path.equals(Environment.getExternalStorageDirectory().getAbsolutePath())) {
-                    internal.setChecked(true);
-                    selected = true;
-                }
-
-                final RadioGroup radiogroup = (RadioGroup) dialog.findViewById(R.id.radiogroup);
-                for (int i = 0; i < extMounts.size(); i++) {
-                    RadioButton myRadioButton = new RadioButton(PreferencesActivity.this);
-                    myRadioButton.setFocusable(true);
-                    myRadioButton.setText(getString(R.string.external_mount) + " @ " + extMounts.get(i));
-                    if(!selected && current_path.equals(extMounts.get(i))) {
-                        myRadioButton.setChecked(true);
-                        selected = true;
-                    }
-                    myRadioButton.setTag(extMounts.get(i));
-                    radiogroup.addView(myRadioButton,i+1); // i+1 to keep "internal memory" on top
-                }
-                radiogroup.invalidate();
-
-                if(!selected) {
-                    other.setChecked(true);
-                    other_path.setEnabled(true);
-                    other_path.setText(current_path);
-                    selected = true;
-                }
-
-                Button button_ok = (Button) dialog.findViewById(R.id.ok);
-                button_ok.setOnClickListener(new Button.OnClickListener() {
-
-                    @Override
-                    public void onClick(View v) {
-                        String newRoot;
-
-                        int radioButtonID = radiogroup.getCheckedRadioButtonId();
-                        RadioButton radioButton = (RadioButton)radiogroup.findViewById(radioButtonID);
-                        if(radioButton.getTag().equals("other")) {
-                            TextView other_path = (TextView)dialog.findViewById(R.id.other_path);
-                            newRoot = other_path.getText().toString();
-                        } else {
-                            newRoot = (String) radioButton.getTag();
-                        }
-
-                        // did it even change? if not, don't bother with rescan.
-                        if(!newRoot.equals(current_path)) {
-                            SharedPreferences.Editor editor = sharedPrefs.edit();
-                            editor.putString("directories_root", newRoot);
-                            editor.commit();
-
-                            setResult(RESULT_RESCAN);
-                        }
-                        dialog.dismiss();
-                    }
-                });
-                Button button_cancel = (Button) dialog.findViewById(R.id.cancel);
-                button_cancel.setOnClickListener(new Button.OnClickListener() {
-                    @Override
-                    public void onClick(View v) { dialog.cancel(); }
-                });
-
-                dialog.show();
-                return true;
-            }});
-
         // Create onClickListen
         Preference directoriesPref = findPreference("directories");
         directoriesPref.setOnPreferenceClickListener(
@@ -243,6 +122,17 @@ public class PreferencesActivity extends PreferenceActivity {
                     }
                 });
 
+        // steal remote control
+        Preference checkboxStealRC = findPreference("steal_remote_control");
+        checkboxStealRC.setOnPreferenceClickListener(
+                new OnPreferenceClickListener() {
+                    @Override
+                    public boolean onPreferenceClick(Preference preference) {
+                        restartService(preference.getContext());
+                        return true;
+                    }
+                });
+
         // Change verbosity (logcat)
         CheckBoxPreference checkboxVerbosity = (CheckBoxPreference) findPreference("enable_verbose_mode");
         checkboxVerbosity.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
@@ -262,6 +152,19 @@ public class PreferencesActivity extends PreferenceActivity {
             }
         });
 
+        // Screen Orientation
+        ListPreference screenOrientationPref = (ListPreference) findPreference("screen_orientation");
+        screenOrientationPref.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
+            @Override
+            public boolean onPreferenceChange(Preference preference, Object newValue) {
+                final SharedPreferences sharedPrefs = PreferenceManager.getDefaultSharedPreferences(PreferencesActivity.this);
+                SharedPreferences.Editor editor = sharedPrefs.edit();
+                editor.putString("screen_orientation_value", (String)newValue);
+                editor.commit();
+                return true;
+            }
+        });
+
         // Audio output
         ListPreference aoutPref = (ListPreference) findPreference("aout");
         int aoutEntriesId = Util.isGingerbreadOrLater() ? R.array.aouts : R.array.aouts_froyo;
@@ -320,4 +223,14 @@ public class PreferencesActivity extends PreferenceActivity {
         super.onPause();
         AudioServiceController.getInstance().unbindAudioService(this);
     }
+
+    private void restartService(Context context) {
+        Intent service = new Intent(context, AudioService.class);
+
+        AudioServiceController.getInstance().unbindAudioService(PreferencesActivity.this);
+        context.stopService(service);
+
+        context.startService(service);
+        AudioServiceController.getInstance().bindAudioService(PreferencesActivity.this);
+    }
 }
