@@ -397,6 +397,13 @@ public class AudioBrowserFragment extends SherlockFragment implements ISortable
         };
     };
 
+    private final Comparator<Media> byMRL = new Comparator<Media>() {
+        @Override
+        public int compare(Media m1, Media m2) {
+            return String.CASE_INSENSITIVE_ORDER.compare(m1.getLocation(), m2.getLocation());
+        };
+    };
+
     private final Comparator<Media> byLength = new Comparator<Media>() {
         @Override
         public int compare(Media m1, Media m2) {
@@ -411,7 +418,7 @@ public class AudioBrowserFragment extends SherlockFragment implements ISortable
         public int compare(Media m1, Media m2) {
             int res = String.CASE_INSENSITIVE_ORDER.compare(m1.getAlbum(), m2.getAlbum());
             if (res == 0)
-                res = byName.compare(m1, m2);
+                res = byMRL.compare(m1, m2);
             return res;
         };
     };
