@@ -94,6 +94,9 @@ public class VideoGridFragment extends SherlockGridFragment implements ISortable
     private ThumbnailerManager mThumbnailerManager;
     private VideoGridAnimator mAnimator;
 
+    // Play video from start or not
+    public boolean mFromStart = false;
+
     /* All subclasses of Fragment must include a public empty constructor. */
     public VideoGridFragment() { }
 
@@ -154,6 +157,7 @@ public class VideoGridFragment extends SherlockGridFragment implements ISortable
         mMediaLibrary.addUpdateHandler(mHandler);
         updateViewMode();
         mAnimator.animate();
+        mFromStart = false;
     }
 
     @Override
@@ -239,7 +243,7 @@ public class VideoGridFragment extends SherlockGridFragment implements ISortable
 
     protected void playVideo(int position) {
         Media item = (Media) getListAdapter().getItem(position);
-        VideoPlayerActivity.start(getActivity(), item.getLocation());
+        VideoPlayerActivity.start(getActivity(), item.getLocation(), mFromStart);
     }
 
     private boolean handleContextItemSelected(MenuItem menu, int position) {
@@ -248,6 +252,10 @@ public class VideoGridFragment extends SherlockGridFragment implements ISortable
         case R.id.video_list_play:
             playVideo(position);
             return true;
+        case R.id.video_list_play_from_start:
+            mFromStart = true;
+            playVideo(position);
+            return true;
         case R.id.video_list_info:
             Intent intent = new Intent(getActivity(), MediaInfoActivity.class);
             intent.putExtra("itemLocation",
@@ -280,7 +288,7 @@ public class VideoGridFragment extends SherlockGridFragment implements ISortable
     @Override
     public boolean onContextItemSelected(MenuItem menu) {
         AdapterContextMenuInfo info = (AdapterContextMenuInfo) menu.getMenuInfo();
-        if (handleContextItemSelected(menu, info.position))
+        if (info != null && handleContextItemSelected(menu, info.position))
             return true;
         return super.onContextItemSelected(menu);
     }
