@@ -106,6 +106,7 @@ public class VideoPlayerActivity extends Activity {
 
     /** Overlay */
     private View mOverlayHeader;
+    private View mOverlayLock;
     private View mOverlayOption;
     private View mOverlay;
     private static final int OVERLAY_TIMEOUT = 4000;
@@ -114,7 +115,6 @@ public class VideoPlayerActivity extends Activity {
     private static final int SHOW_PROGRESS = 2;
     private static final int SURFACE_SIZE = 3;
     private static final int FADE_OUT_INFO = 4;
-    private static final int HIDE_NAV = 5;
     private boolean mDragging;
     private boolean mShowing;
     private int mUiVisibility = -1;
@@ -128,11 +128,14 @@ public class VideoPlayerActivity extends Activity {
     private IPlayerControl mControls;
     private boolean mEnableWheelbar;
     private boolean mEnableBrightnessGesture;
+    private boolean mDisplayRemainingTime = false;
+    private int mScreenOrientation;
     private ImageButton mAudioTrack;
     private ImageButton mSubtitle;
     private ImageButton mLock;
     private ImageButton mSize;
     private TextView mSpeedLabel;
+    private boolean mIsLocked = false;
 
     /**
      * For uninterrupted switching between audio and video mode
@@ -184,7 +187,6 @@ public class VideoPlayerActivity extends Activity {
                             setSurfaceSize(mVideoWidth, mVideoHeight, mSarNum, mSarDen);
                             if (visibility == View.SYSTEM_UI_FLAG_VISIBLE && !mShowing) {
                                 showOverlay();
-                                mHandler.sendMessageDelayed(mHandler.obtainMessage(HIDE_NAV), OVERLAY_TIMEOUT);
                             }
                             mUiVisibility = visibility;
                         }
@@ -193,6 +195,7 @@ public class VideoPlayerActivity extends Activity {
 
         /** initialize Views an their Events */
         mOverlayHeader = findViewById(R.id.player_overlay_header);
+        mOverlayLock = findViewById(R.id.lock_overlay);
         mOverlayOption = findViewById(R.id.option_overlay);
         mOverlay = findViewById(R.id.player_overlay);
 
@@ -201,13 +204,20 @@ public class VideoPlayerActivity extends Activity {
         mSysTime = (TextView) findViewById(R.id.player_overlay_systime);
         mBattery = (TextView) findViewById(R.id.player_overlay_battery);
 
+        // Position and remaining time
         mTime = (TextView) findViewById(R.id.player_overlay_time);
+        mTime.setOnClickListener(mRemainingTimeListener);
         mLength = (TextView) findViewById(R.id.player_overlay_length);
+        mLength.setOnClickListener(mRemainingTimeListener);
+
         // the info textView is not on the overlay
         mInfo = (TextView) findViewById(R.id.player_overlay_info);
 
         mEnableWheelbar = pref.getBoolean("enable_wheel_bar", false);
         mEnableBrightnessGesture = pref.getBoolean("enable_gesture_brightness", true);
+        mScreenOrientation = Integer.valueOf(
+                pref.getString("screen_orientation_value", "4" /*SCREEN_ORIENTATION_SENSOR*/));
+
         mControls = mEnableWheelbar
                 ? new PlayerControlWheel(this)
                 : new PlayerControlClassic(this);
@@ -231,7 +241,7 @@ public class VideoPlayerActivity extends Activity {
             }}, 1500);
 
 
-        mLock = (ImageButton) findViewById(R.id.player_overlay_lock);
+        mLock = (ImageButton) findViewById(R.id.lock_overlay_button);
         mLock.setOnClickListener(mLockListener);
 
         mSize = (ImageButton) findViewById(R.id.player_overlay_size);
@@ -260,7 +270,8 @@ public class VideoPlayerActivity extends Activity {
             LibVLC.useIOMX(this);
             mLibVLC = LibVLC.getInstance();
         } catch (LibVlcException e) {
-            e.printStackTrace();
+            Log.d(TAG, "LibVLC initialisation failed");
+            return;
         }
 
         EventManager em = EventManager.getInstance();
@@ -268,13 +279,16 @@ public class VideoPlayerActivity extends Activity {
 
         this.setVolumeControlStream(AudioManager.STREAM_MUSIC);
 
-        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_SENSOR);
+        // 100 is the value for screen_orientation_start_lock
+        setRequestedOrientation(mScreenOrientation != 100
+                ? mScreenOrientation
+                : getScreenOrientation());
     }
 
     @Override
     protected void onStart() {
         super.onStart();
-        dimStatusBar(true);
+        showOverlay();
         mSwitchingView = false;
     }
 
@@ -282,8 +296,13 @@ public class VideoPlayerActivity extends Activity {
     protected void onPause() {
         super.onPause();
 
-        if(mSwitchingView)
+        if(mSwitchingView) {
+            Log.d(TAG, "mLocation = \"" + mLocation + "\"");
+            AudioServiceController.getInstance().showWithoutParse(mLocation);
+            AudioServiceController.getInstance().unbindAudioService(this);
+            AudioPlayerActivity.start(this, true);
             return;
+        }
 
         long time = mLibVLC.getTime();
         long length = mLibVLC.getLength();
@@ -323,6 +342,8 @@ public class VideoPlayerActivity extends Activity {
                     DatabaseManager.mediaColumn.MEDIA_TIME,
                     time);
         }
+
+        AudioServiceController.getInstance().unbindAudioService(this);
     }
 
     @Override
@@ -337,13 +358,6 @@ public class VideoPlayerActivity extends Activity {
         em.removeHandler(eventHandler);
 
         mAudioManager = null;
-
-        if(mSwitchingView) {
-            Log.d(TAG, "mLocation = \"" + mLocation + "\"");
-            AudioServiceController.getInstance().showWithoutParse(mLocation);
-            AudioPlayerActivity.start(this, true);
-        }
-        //AudioServiceController.getInstance().unbindAudioService(this);
     }
 
     @Override
@@ -373,14 +387,23 @@ public class VideoPlayerActivity extends Activity {
     }
 
     public static void start(Context context, String location) {
-        start(context, location, null, false);
+        start(context, location, null, false, false);
+    }
+
+    public static void start(Context context, String location, Boolean fromStart) {
+        start(context, location, null, false, fromStart);
     }
 
     public static void start(Context context, String location, String title, Boolean dontParse) {
+        start(context, location, title, dontParse, false);
+    }
+
+    public static void start(Context context, String location, String title, Boolean dontParse, Boolean fromStart) {
         Intent intent = new Intent(context, VideoPlayerActivity.class);
         intent.putExtra("itemLocation", location);
         intent.putExtra("itemTitle", title);
         intent.putExtra("dontParse", dontParse);
+        intent.putExtra("fromStart", fromStart);
 
         if (dontParse)
             intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_MULTIPLE_TASK);
@@ -434,45 +457,24 @@ public class VideoPlayerActivity extends Activity {
     /**
      * Lock screen rotation
      */
-    @SuppressWarnings("deprecation")
     private void lockScreen() {
-        WindowManager wm = (WindowManager) getSystemService(WINDOW_SERVICE);
-        Display display = wm.getDefaultDisplay();
-        int rotation;
-        if (Build.VERSION.SDK_INT >= 8 /* Android 2.2 has getRotation */) {
-            try {
-                Method m = display.getClass().getDeclaredMethod("getRotation");
-                rotation = (Integer) m.invoke(display);
-            } catch (Exception e) {
-                rotation = Surface.ROTATION_0;
-            }
-        } else {
-            rotation = display.getOrientation();
-        }
-        switch (rotation) {
-            case Surface.ROTATION_0:
-                setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
-                break;
-            case Surface.ROTATION_90:
-                setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
-                break;
-            case Surface.ROTATION_270:
-                // FIXME: API Level 9+ (not tested on a device with API Level < 9)
-                setRequestedOrientation(8); // SCREEN_ORIENTATION_REVERSE_LANDSCAPE
-                break;
-        }
-
+        if(mScreenOrientation == ActivityInfo.SCREEN_ORIENTATION_SENSOR)
+            setRequestedOrientation(getScreenOrientation());
         showInfo(R.string.locked, 1000);
         mLock.setBackgroundResource(R.drawable.ic_lock_glow);
+        hideOverlay(true);
     }
 
     /**
      * Remove screen lock
      */
     private void unlockScreen() {
-        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_SENSOR);
+        if(mScreenOrientation == ActivityInfo.SCREEN_ORIENTATION_SENSOR)
+            setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_SENSOR);
         showInfo(R.string.unlocked, 1000);
         mLock.setBackgroundResource(R.drawable.ic_lock);
+        mShowing = false;
+        showOverlay();
     }
 
     /**
@@ -599,9 +601,6 @@ public class VideoPlayerActivity extends Activity {
                 case FADE_OUT_INFO:
                     activity.fadeOutInfo();
                     break;
-                case HIDE_NAV:
-                    activity.dimStatusBar(true);
-                    break;
             }
         }
     };
@@ -705,6 +704,11 @@ public class VideoPlayerActivity extends Activity {
 
     @Override
     public boolean onTouchEvent(MotionEvent event) {
+        if (mIsLocked) {
+            showOverlay();
+            return false;
+        }
+
         DisplayMetrics screen = new DisplayMetrics();
         getWindowManager().getDefaultDisplay().getMetrics(screen);
 
@@ -752,10 +756,6 @@ public class VideoPlayerActivity extends Activity {
                     showOverlay();
                 } else {
                     hideOverlay(true);
-                    if (Util.isICSOrLater())
-                        mHandler.sendMessageDelayed(
-                                mHandler.obtainMessage(HIDE_NAV),
-                                OVERLAY_TIMEOUT);
                 }
             }
             // Seek
@@ -981,18 +981,16 @@ public class VideoPlayerActivity extends Activity {
      *
      */
     private final OnClickListener mLockListener = new OnClickListener() {
-        boolean isLocked = false;
 
         @Override
         public void onClick(View v) {
-            if (isLocked) {
+            if (mIsLocked) {
+                mIsLocked = false;
                 unlockScreen();
-                isLocked = false;
             } else {
+                mIsLocked = true;
                 lockScreen();
-                isLocked = true;
             }
-            showOverlay();
         }
     };
 
@@ -1051,6 +1049,14 @@ public class VideoPlayerActivity extends Activity {
         }
     };
 
+    private final OnClickListener mRemainingTimeListener = new OnClickListener() {
+        @Override
+        public void onClick(View v) {
+            mDisplayRemainingTime = !mDisplayRemainingTime;
+            showOverlay();
+        }
+    };
+
     /**
      * attach and disattach surface to the lib
      */
@@ -1081,14 +1087,16 @@ public class VideoPlayerActivity extends Activity {
      * show overlay
      */
     private void showOverlay(int timeout) {
-        mHandler.removeMessages(HIDE_NAV);
         mHandler.sendEmptyMessage(SHOW_PROGRESS);
         if (!mShowing) {
             mShowing = true;
-            mOverlayHeader.setVisibility(View.VISIBLE);
-            mOverlayOption.setVisibility(View.VISIBLE);
-            mOverlay.setVisibility(View.VISIBLE);
-            dimStatusBar(false);
+            mOverlayLock.setVisibility(View.VISIBLE);
+            if (!mIsLocked) {
+                mOverlayHeader.setVisibility(View.VISIBLE);
+                mOverlayOption.setVisibility(View.VISIBLE);
+                mOverlay.setVisibility(View.VISIBLE);
+                dimStatusBar(false);
+            }
         }
         Message msg = mHandler.obtainMessage(FADE_OUT);
         if (timeout != 0) {
@@ -1106,11 +1114,13 @@ public class VideoPlayerActivity extends Activity {
         if (mShowing) {
             mHandler.removeMessages(SHOW_PROGRESS);
             Log.i(TAG, "remove View!");
-            if (!fromUser) {
+            if (!fromUser && !mIsLocked) {
+                mOverlayLock.startAnimation(AnimationUtils.loadAnimation(this, android.R.anim.fade_out));
                 mOverlayHeader.startAnimation(AnimationUtils.loadAnimation(this, android.R.anim.fade_out));
                 mOverlayOption.startAnimation(AnimationUtils.loadAnimation(this, android.R.anim.fade_out));
                 mOverlay.startAnimation(AnimationUtils.loadAnimation(this, android.R.anim.fade_out));
             }
+            mOverlayLock.setVisibility(View.INVISIBLE);
             mOverlayHeader.setVisibility(View.INVISIBLE);
             mOverlayOption.setVisibility(View.INVISIBLE);
             mOverlay.setVisibility(View.INVISIBLE);
@@ -1127,11 +1137,12 @@ public class VideoPlayerActivity extends Activity {
     private void dimStatusBar(boolean dim) {
         if (!Util.isHoneycombOrLater())
             return;
-
         mSurface.setSystemUiVisibility(
-                dim ? (Util.hasNavBar()
-                        ? View.SYSTEM_UI_FLAG_HIDE_NAVIGATION|View.SYSTEM_UI_FLAG_LOW_PROFILE
-                        : View.SYSTEM_UI_FLAG_LOW_PROFILE)
+                dim ? (Util.hasCombBar()
+                        ? View.SYSTEM_UI_FLAG_LOW_PROFILE
+                        : Util.hasNavBar()
+                            ? View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
+                            : View.SYSTEM_UI_FLAG_LOW_PROFILE)
                     : View.SYSTEM_UI_FLAG_VISIBLE);
     }
 
@@ -1158,7 +1169,9 @@ public class VideoPlayerActivity extends Activity {
         mSeekbar.setProgress(time);
         mSysTime.setText(DateFormat.format("kk:mm", System.currentTimeMillis()));
         mTime.setText(Util.millisToString(time));
-        mLength.setText(Util.millisToString(length));
+        mLength.setText(mDisplayRemainingTime && length > 0
+                ? "- " + Util.millisToString(length - time)
+                : Util.millisToString(length));
         return time;
     }
 
@@ -1214,6 +1227,7 @@ public class VideoPlayerActivity extends Activity {
         mLocation = null;
         String title = getResources().getString(R.string.title);
         boolean dontParse = false;
+        boolean fromStart = false;
         String itemTitle = null;
 
         if (getIntent().getAction() != null
@@ -1225,6 +1239,7 @@ public class VideoPlayerActivity extends Activity {
             mLocation = getIntent().getExtras().getString("itemLocation");
             itemTitle = getIntent().getExtras().getString("itemTitle");
             dontParse = getIntent().getExtras().getBoolean("dontParse");
+            fromStart = getIntent().getExtras().getBoolean("fromStart");
         }
 
         mSurface.setKeepScreenOn(true);
@@ -1239,7 +1254,7 @@ public class VideoPlayerActivity extends Activity {
         if (mLocation != null && mLocation.length() > 0 && !dontParse) {
             // restore last position
             Media media = DatabaseManager.getInstance(this).getMedia(this, mLocation);
-            if (media != null && media.getTime() > 0)
+            if (media != null && media.getTime() > 0 && !fromStart)
                 mLibVLC.setTime(media.getTime());
 
             try {
@@ -1257,4 +1272,41 @@ public class VideoPlayerActivity extends Activity {
         }
         mTitle.setText(title);
     }
+
+    @SuppressWarnings("deprecation")
+    private int getScreenRotation(){
+        WindowManager wm = (WindowManager) getSystemService(Context.WINDOW_SERVICE);
+        Display display = wm.getDefaultDisplay();
+        if (Build.VERSION.SDK_INT >= 8 /* Android 2.2 has getRotation */) {
+            try {
+                Method m = display.getClass().getDeclaredMethod("getRotation");
+                return (Integer) m.invoke(display);
+            } catch (Exception e) {
+                return Surface.ROTATION_0;
+            }
+        } else {
+            return display.getOrientation();
+        }
+    }
+
+    private int getScreenOrientation (){
+        switch (getScreenRotation()) {
+        case Surface.ROTATION_0:
+            return ActivityInfo.SCREEN_ORIENTATION_PORTRAIT;
+        case Surface.ROTATION_90:
+            return ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE;
+        case Surface.ROTATION_180:
+            // SCREEN_ORIENTATION_REVERSE_PORTRAIT only available since API Level 9+
+             return (Build.VERSION.SDK_INT >= 8
+                    ? ActivityInfo.SCREEN_ORIENTATION_REVERSE_PORTRAIT
+                    : ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
+        case Surface.ROTATION_270:
+            // SCREEN_ORIENTATION_REVERSE_LANDSCAPE only available since API Level 9+
+            return (Build.VERSION.SDK_INT >= 8
+                    ? ActivityInfo.SCREEN_ORIENTATION_REVERSE_LANDSCAPE
+                    : ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
+        default :
+            return 0;
+        }
+    }
 }
